/**
 * Contains methods for interaction with Infusionsoft
 */

const infusionsoftAPI = (function () {
  return {
    requestUnauthorizedMessage: '401',
    noValidTokenStoredMessage: 'No valid access token stored',
    refreshTokenIsExpiredMessage: 'Refresh token is expired',
    noContactIsFoundMessage: 'No contact is found',

    _xmlrpc: function (accessToken, methodName, params) {
      return $.xmlrpc({
        url: 'https://api.infusionsoft.com/crm/xmlrpc/v1?access_token=' + accessToken.access_token,
        methodName: methodName,
        params: params
      });
    },

    xmlrpcWithRetry: function (accessToken, methodName, params) {
      return this._xmlrpc(accessToken, methodName, params).catch(jqXHR => {
        if (jqXHR.status === 401) {
          return this._refreshAccessToken(accessToken.refresh_token)
            .then(refreshedAccessToken => this._xmlrpc(refreshedAccessToken, methodName, params));
        } else {
          throw jqXHR.statusText || 'Error';
        }
      });
    },

    _ajax: function (accessToken, endpoint, data) {
      return $.ajax({
        url: window.oauth2.api_url + endpoint,
        type: 'POST',
        dataType: 'json',
        headers: {
          'Authorization': 'Bearer ' + accessToken.access_token,
          'Accept': 'application/json',
          'Content-Type': 'application/json',
        },
        data: data
      });
    },

    ajaxWithRetry: function (accessToken, endpoint, data) {
      return this._ajax(accessToken, endpoint, data).catch(jqXHR => {
        if (jqXHR.status === 401) {
          return this._refreshAccessToken(accessToken.refresh_token).then(refreshedAccessToken => this._ajax(refreshedAccessToken, endpoint, data));
        } else {
          throw jqXHR.statusText || 'Error';
        }
      });
    },

    _getAccessTokenFromStorage: function (account) {
      return new Promise((resolve, reject) => {
        chrome.storage.local.get('tokens', function (tokens) {
          if (tokens.tokens !== undefined) {
            tokens = tokens.tokens;
            let accessToken = tokens[account];
            if (accessToken !== undefined) {
              resolve(accessToken);
            } else {
              reject(this.noValidTokenStoredMessage);
            }
          } else {
            reject(this.noValidTokenStoredMessage);
          }
        }.bind(this));
      });
    },

    getUserId: function (account) {
      return this._getAccessTokenFromStorage(account)
        .then(accessToken => accessToken.userId);
    },

    getValidAccessToken: function (account) {
      return this._getAccessTokenFromStorage(account).then(accessToken => {
        let timestamp = Math.round(new Date() / 1000);
        if ((accessToken.expires_in - timestamp) < 3 * 60 * 60) { // 3 hours is recommended by Infusionsoft
          if ((accessToken.refresh_token_expires_in - timestamp) > 0) { // refresh token is not expired
            return this._refreshAccessToken(accessToken.refresh_token);
          } else {
            throw this.refreshTokenIsExpiredMessage;
          }
        } else {
          return accessToken;
        }
      });
    },

    _refreshAccessToken: function (refreshToken) {
      return window.oauth2.refreshAccessToken(refreshToken)
        .then(tokenResponse => this.processAccessToken(tokenResponse))
        .then(accessToken => {
          this.storeAccessTokenInStorage(accessToken);
          return accessToken;
        });
    },

    processAccessToken: function (accessToken) {
      return this._getUserInfo(accessToken.access_token)
        .then(userInfo => this._getUserIdByEmail(accessToken, userInfo.email))
        .then(userId => {
          accessToken.userId = userId;
          const timestamp = Math.round(new Date() / 1000);
          accessToken.expires_in += timestamp;
          accessToken.refresh_token_expires_in = timestamp + 180 * 24 * 60 * 60; // 180 days

          return accessToken;
        });
    },

    storeAccessTokenInStorage: function (accessToken) {
      chrome.storage.local.get('tokens', function (tokens) {
        tokens = tokens.tokens || {};
        let account = accessToken.scope.split('|')[1];
        tokens[account] = accessToken;
        chrome.storage.local.set({'tokens': tokens}, function () {
          console.log('access token is saved in storage');
        });
      });
    },

    _getUserInfo: function (accessToken) {
      return $.ajax({
        url: window.oauth2.api_url + '/oauth/connect/userinfo',
        type: 'GET',
        dataType: 'json',
        data: {
          access_token: accessToken
        }
      });
    },

    _getUserIdByEmail: function (accessToken, email) {
      return this._xmlrpc(
        accessToken,
        'DataService.findByField',
        [
          window.oauth2.client_id,
          'User',
          1,
          0,
          'Email',
          email,
          ['Id'],
        ]
      ).then(userResponse => {
        if (userResponse[0].length > 0) {
          return userResponse[0][0].Id;
        } else {
          throw 'user response is in incorrect format';
        }
      });
    },
  };
})();
